﻿using Aspose.Words;
using Aspose.Words.Drawing;
using Aspose.Words.Replacing;
using Aspose.Words.Tables;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Packaging;
using Labs.HelperClasses;
using Labs.HelperClasses.WordTable;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Labs
{
    public partial class WordTable : Component
    {
        public WordTable()
        {
            InitializeComponent();
        }

        public WordTable(IContainer container)
        {
            container.Add(this);

            InitializeComponent();
        }

        public void CreateWordTable<T>(WordTableConfig<T> config)
        {
            if (config == null)
            {
                return;
            }

            //проверка на заполненность данных о столбцах (заголовок, хранящееся св-во, ширина)
            foreach (var colProp in config.ColumnsProperties)
            {
                if (colProp.Header1 == null)
                {
                    return;
                }
                if (string.IsNullOrEmpty(colProp.PropertyName))
                {
                    throw new ArgumentException($"Не задано свойство для столбца");
                }
                if (colProp.Width <= 0)
                {
                    throw new ArgumentException($"Неверная ширина для столбца");
                }
            }

            //Проверка на то, что объединённые ячейки не пересекаются
            if (config.MergedCells.Count() >= 2)
            {
                var sortedCells = config.MergedCells.OrderBy(cell => cell.StartCell).ToList();
                for (int i = 1; i < sortedCells.Count; i++)
                {
                    // если текущий StartCell меньше или равен предыдущему EndCell, то есть пересечение
                    if (sortedCells[i].StartCell <= sortedCells[i - 1].EndCell)
                    {
                        throw new ArgumentException("Объединённые ячейки пересекаются");
                    }
                }
            }

            // Создание пустого word файла
            Document doc = new Document();
            DocumentBuilder builder = new DocumentBuilder(doc);

            builder.PageSetup.Orientation = Aspose.Words.Orientation.Landscape;

            //стиль для заголовка
            Style titleStyle = builder.Document.Styles.Add(StyleType.Paragraph, "Title");
            titleStyle.Font.Size = 16;
            titleStyle.Font.Bold = true;
            titleStyle.ParagraphFormat.Alignment = ParagraphAlignment.Left;
            titleStyle.ParagraphFormat.SpaceAfter = 12; 
            //titleStyle.ParagraphFormat.WordWrap = true;

            //написание заголовка первой строкой
            builder.ParagraphFormat.Style = titleStyle;
            builder.Writeln(config.DocumentTitle);

            #region СОЗДАНИЕ САМОЙ ТАБЛИЦЫ
            Table table = builder.StartTable();

            builder.ParagraphFormat.Alignment = ParagraphAlignment.Center;

            #region ПЕРВАЯ СТРОКА ЗАГОЛОВКОВ
            //вставка ячеек 1-й строки шапки
            foreach (var colProp in config.ColumnsProperties)
            {
                builder.InsertCell();
                builder.CellFormat.Width = colProp.Width;
                builder.CellFormat.WrapText = true;
                //builder.CellFormat.PreferredWidth = PreferredWidth.FromPoints(colProp.Width);

                //выравнивания
                builder.ParagraphFormat.Alignment = ParagraphAlignment.Center;
                builder.CellFormat.VerticalAlignment = CellVerticalAlignment.Center;
                // Устанавливаем стиль текста
                builder.Font.Size = 14; // Размер текста
                builder.Font.Bold = true; // Жирный текст
                //МБ Writeln
                builder.Write(colProp.Header1);
            }


            //объединение ячеек в 1 строке
            foreach (var mergedCell in config.MergedCells)
            {
                int startCellIndex = mergedCell.StartCell;
                int endCellIndex = mergedCell.EndCell;

                //Начало объединения
                table.Rows[0].Cells[startCellIndex].CellFormat.HorizontalMerge = CellMerge.First;
                //Все след. ячейки объединяются последовательно с предыдущей
                for (int i = startCellIndex+1; i <= endCellIndex; i++)
                {
                    table.Rows[0].Cells[i].CellFormat.HorizontalMerge = CellMerge.Previous;
                }
            }
            //переход на след. строку
            builder.EndRow();
            #endregion

            #region ВТОРАЯ СТРОКА ЗАГОЛОВКОВ

            int index = 0;
            //вставка ячеек 2-й строки шапки
            foreach (var colProp in config.ColumnsProperties)
            {
                builder.InsertCell();
                builder.CellFormat.Width = colProp.Width;
                //builder.CellFormat.PreferredWidth = PreferredWidth.FromPoints(colProp.Width);

                //выравнивания
                //builder.ParagraphFormat.Alignment = ParagraphAlignment.Center;
                //builder.CellFormat.VerticalAlignment = CellVerticalAlignment.Center;

                
                //Если ячейка сверху не объединена по горизонтали (в 1 строке), объединить с ней
                if (table.Rows[0].Cells[index].CellFormat.HorizontalMerge == CellMerge.None)
                {
                    table.Rows[0].Cells[index].CellFormat.VerticalMerge = CellMerge.First;
                    table.Rows[1].Cells[index].CellFormat.VerticalMerge = CellMerge.Previous;
                }
                else
                {
                    builder.Write(colProp.Header2);
                }
                index++;
            }
            //переход на след. строку
            builder.EndRow();
            builder.EndTable();
            #endregion

            #region ОСНОВНАЯ ЧАСТЬ ТАБЛИЦЫ (С ДАННЫМИ)

            table = builder.StartTable();
            builder.Font.Bold = false;
            //для каждой записи (1 объекта)
            foreach (var rowData in config.Data)
            {
                //для каждого столбца
                foreach (var colProp in config.ColumnsProperties)
                {
                    //свойство класса, которое хранится в столбце
                    builder.InsertCell();
                    builder.CellFormat.Width = colProp.Width;
                    builder.CellFormat.WrapText = true;
                    var propertyValue = rowData.GetType()
                        .GetProperty(colProp.PropertyName)?
                        .GetValue(rowData)?.ToString();

                    builder.Write(propertyValue ?? "");
                }
                //переход на новую строку
                builder.EndRow();
            }
            builder.EndTable();
            #endregion

            #endregion
            //сохранение док-а
            doc.Save(config.FilePath);

            new Remover().Remove(config.FilePath);

        }
        private void RemoveDrawings(IEnumerable<DocumentFormat.OpenXml.Wordprocessing.Drawing> drawings)
        {

        }
    }
}
